import java.awt.*;

public class DrawningObjectPlane implements IDrawningObject
{
    private DrawingPlane _plane = null;

    public DrawingPlane GetDrawingObjectPlane()
    {
        return _plane;
    }

    public DrawningObjectPlane(DrawingPlane plane){ _plane = plane; }

    public DrawingPlane GetPlane()
    {
        return _plane;
    }

    @Override
    public float Step()
    {
        if(_plane != null && _plane.Plane != null)
        {
            return _plane.Plane.GetStep();
        }

        return 0;
    }

    @Override
    public void SetObject(int x, int y, int width, int height)
    {
        _plane.SetPosition(x, y, width, height);
    }

    @Override
    public void MoveObject(Direction direction)
    {
        _plane.MoveTransport(direction);
    }

    @Override
    public void DrawningObject(Graphics g)
    {
        _plane.DrawTransport(g);
    }

    @Override
    public float[] GetCurrentPosition()
    {
        if(_plane != null)
        {
            return _plane.GetCurrentPosition();
        }

        return null;
    }

    public String GetInfo()
    {
        if(_plane != null)
        {
            return ExtentionPlane.GetDataForSave(_plane);
        }
        return null;
    }

    @Override
    public boolean equals(Object obj)
    {
        if(obj == null)
        {
            return false;
        }

        var otherPlane =  (DrawningObjectPlane) obj;

        if(otherPlane == null)
        {
            return false;
        }

        var plane = _plane.Plane;
        var otherPlanePlane = otherPlane._plane.Plane;

        if(plane.GetSpeed() != otherPlanePlane.GetSpeed())
        {
            return false;
        }

        if(plane.GetWeight() != otherPlanePlane.GetWeight())
        {
            return false;
        }

        if(plane.GetColor().getRGB() != otherPlanePlane.GetColor().getRGB())
        {
            return false;
        }

        if(plane.getClass().getSimpleName() != otherPlanePlane.getClass().getSimpleName())
        {
            return false;
        }

        if (plane instanceof EntityAirbus entitySuperAirbus && otherPlanePlane instanceof EntityAirbus otherEntitySuperAirbus)
        {
            if(entitySuperAirbus.AddEngine() != otherEntitySuperAirbus.AddEngine())
            {
                return false;
            }

            if(entitySuperAirbus.AddCompartment() != otherEntitySuperAirbus.AddCompartment())
            {
                return false;
            }

            return entitySuperAirbus.AddColor().getRGB() == otherEntitySuperAirbus.AddColor().getRGB();
        }

        return true;
    }

    public static IDrawningObject Create(String data)
    {
        return new DrawningObjectPlane(ExtentionPlane.CreateDrawingPlane(data));
    }
}
