import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.util.HashMap;

public class FormMapWithSetPlanesGeneric extends JFrame{
    public JPanel MainPanel;
    private JPanel PictureBoxPlane;
    private JPanel GroupBoxTools;
    private JComboBox ComboBoxSelectorMap;
    private JButton ButtonLeft;
    private JButton ButtonDown;
    private JButton ButtonRight;
    private JButton ButtonUp;
    private JButton ButtonShowOnMap;
    private JButton ButtonShowStorage;
    private JButton ButtonAddPlane;
    private JTextField MaskedTextBoxPosition;
    private JButton ButtonRemovePlane;
    private JTextField TextBoxNewMapName;
    private JButton ButtonAddMap;
    private JList ListBoxMaps;
    private JButton ButtonDeleteMap;
    private JPanel GroupBoxMap;
    private JButton ButtonShowDelete;
    private JMenuBar MenuBarSaveLoad;
    private JMenu MenuChoiceOperation;
    private JMenuItem MenuItemLoadData;
    private JMenuItem MenuItemSaveData;
    private JMenuItem MenuItemLoadOneMapData;
    private JMenuItem MenuItemSaveOneMapData;
    private JButton ButtonSortByType;
    private JButton ButtonSortByColor;

    //объект от коллекции карт
    private final MapsCollection _mapsCollection;

    //логгер
    private static Logger _logger;

    //дополнительные логгеры
    Logger logwarn = Logger.getLogger("FormMapWithSetPlanesGenericAdmin");
    Logger loginfo = Logger.getLogger("FormMapWithSetPlanesGeneric");

    //Для выпадающего списка
    HashMap<String, AbstractMap> _mapsHashMap = new HashMap<>()
    {{
            put("Простая карта", new SimpleMap());
            put("Буря в пустыне", new DesertStormMap());
            put("Звёздные войны", new StarWarsMap());
    }};

    public FormMapWithSetPlanesGeneric()
    {
        super("Хранилище");
        Logger.getLogger(FormMapWithSetPlanesGeneric.class.getName());
        CreateWindow();
        _mapsCollection = new MapsCollection(730, 700);
        ComboBoxSelectorMap.removeAllItems();

        for (String elem : _mapsHashMap.keySet())
        {
            ComboBoxSelectorMap.addItem(elem);
        }
    }

    //заполнение ListBoxMaps
    private void ReloadMaps()
    {
        int index = ListBoxMaps.getSelectedIndex();

        DefaultListModel<String> model = (DefaultListModel<String>)ListBoxMaps.getModel();
        model.removeAllElements();

        for (int i = 0; i < _mapsCollection.Keys().size(); i++)
        {
            model.addElement(_mapsCollection.Keys().get(i));
        }

        if (ListBoxMaps.getModel().getSize() > 0 && (index == -1 || index >= ListBoxMaps.getModel().getSize()))
        {
            ListBoxMaps.setSelectedIndex(0);
        }
        else if (ListBoxMaps.getModel().getSize() > 0 && index > -1 && index < ListBoxMaps.getModel().getSize())
        {
            ListBoxMaps.setSelectedIndex(index);
        }
    }

    //обновление отрисовки
    public void UpdateWindow(BufferedImage bmp)
    {
        PictureBoxPlane.removeAll();
        JLabel imageWithMapAndObject = new JLabel();
        imageWithMapAndObject.setPreferredSize(PictureBoxPlane.getSize());
        imageWithMapAndObject.setMinimumSize(new Dimension(1, 1));
        imageWithMapAndObject.setIcon(new ImageIcon(bmp));
        PictureBoxPlane.add(imageWithMapAndObject, BorderLayout.CENTER);
        PictureBoxPlane.revalidate();
    }

    public void CreateWindow()
    {
        //загрузка изображений для кнопок
        try
        {
            Image img = ImageIO.read(getClass().getResource("resourses/Up.png"));
            ButtonUp.setIcon(new ImageIcon(img));
            img = ImageIO.read(getClass().getResource("resourses/Left.png"));
            ButtonLeft.setIcon(new ImageIcon(img));
            img = ImageIO.read(getClass().getResource("resourses/Down.png"));
            ButtonDown.setIcon(new ImageIcon(img));
            img = ImageIO.read(getClass().getResource("resourses/Right.png"));
            ButtonRight.setIcon(new ImageIcon(img));
        }
        catch (Exception ex)
        {
            System.out.println(ex.getMessage());
        }

        //добавить карту
        ButtonAddMap.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(ComboBoxSelectorMap.getSelectedIndex() == -1 || TextBoxNewMapName.getText().isEmpty())
                {
                    JOptionPane.showMessageDialog(null, "Не все данные заполнены", "Ошибка", JOptionPane.ERROR_MESSAGE);

                    if(ComboBoxSelectorMap.getSelectedIndex() == -1)
                    {
                        logwarn.log(Level.ERROR, "При добавлении карты " + ComboBoxSelectorMap.getSelectedIndex() + "не была выбрана карта");
                    }
                    else
                    {
                        logwarn.log(Level.ERROR, "При добавлении карты " + ComboBoxSelectorMap.getSelectedIndex() + "не была названа карта");
                    }

                    return;
                }

                if(!_mapsHashMap.containsKey(ComboBoxSelectorMap.getSelectedItem()))
                {
                    JOptionPane.showMessageDialog(null, "Данная карта отсутсвует", "Ошибка", JOptionPane.ERROR_MESSAGE);
                    logwarn.log(Level.ERROR, "Отсутствует карта с названием " + TextBoxNewMapName.getText());

                    return;
                }

                _mapsCollection.AddMap(TextBoxNewMapName.getText(), _mapsHashMap.get(ComboBoxSelectorMap.getSelectedItem().toString()));
                loginfo.log(Level.INFO, "Добавлена карта " + TextBoxNewMapName.getText());
                ReloadMaps();
            }
        });

        //удалить карту
        ButtonDeleteMap.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(ListBoxMaps.getSelectedIndex() == -1)
                {
                    return;
                }

                if(JOptionPane.showConfirmDialog(null, "Удалить карту" + ListBoxMaps.getSelectedValue().toString() + "?",
                        "Удаление", JOptionPane.YES_NO_OPTION) == 0)
                {
                    _mapsCollection.DelMap(ListBoxMaps.getSelectedValue().toString());
                    loginfo.log(Level.INFO, "Удалена карта " + ListBoxMaps.getSelectedValue().toString());
                    ReloadMaps();
                }
            }
        });

        //список карт
        ListBoxMaps.addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
                if(ListBoxMaps.getSelectedIndex() == -1)
                {
                    return;
                }

                UpdateWindow(_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet());
                loginfo.log(Level.INFO, "Осуществлён переход на карту под названием " + ListBoxMaps.getSelectedValue().toString());
            }
        });

        //Просмотр удалённых самолётов
        ButtonShowDelete.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(ListBoxMaps.getSelectedIndex() == -1)
                {
                    return;
                }

                DrawningObjectPlane plane = (DrawningObjectPlane)_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).GetPlaneInDelete();

                if(plane == null)
                {
                    JOptionPane.showMessageDialog(null, "Список пуст", "Ошибка", JOptionPane.ERROR_MESSAGE);

                    return;
                }

                FormPlane form = new FormPlane(plane);
                form.setSize(700, 500);
                form.setVisible(true);
            }
        });

        //добавление самолёта с использованием делегата
        ButtonAddPlane.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                FormPlaneConfig form = new FormPlaneConfig();

                form.AddEvent(newPlane -> {
                    try
                    {
                        if (ListBoxMaps.getSelectedIndex() == -1) {
                            return;
                        }

                        if (newPlane != null) {
                            DrawningObjectPlane plane = new DrawningObjectPlane(newPlane);

                            if (_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).Add(plane) != -1) {
                                JOptionPane.showMessageDialog(null, "Объект добавлен");
                                loginfo.log(Level.INFO, "Добавлен объект: " + plane);
                                UpdateWindow(_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet());
                            } else {
                                JOptionPane.showMessageDialog(null, "Не удалось добавить объект", "Ошибка", JOptionPane.ERROR_MESSAGE);
                                loginfo.log(Level.INFO, "Не удалось добавить объект: " + plane);
                            }
                        }
                    }
                    catch(StorageOverflowException ex)
                    {
                        JOptionPane.showConfirmDialog(null, "Ошибка, хранилище переполнено: " + ex.getMessage(), "Результат",
                                JOptionPane.ERROR_MESSAGE);
                        logwarn.log(Level.WARN, "Ошибка, переполнение хранилища: " + ex.getMessage());
                    }
                    catch(IllegalArgumentException ex)
                    {
                        JOptionPane.showConfirmDialog(null, ex.getMessage(), "Результат",
                                JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);
                        logwarn.log(Level.ERROR, "Ошибка добавления: " + ex.getMessage());
                    }
                    catch(Exception ex)
                    {
                        JOptionPane.showConfirmDialog(null, "Неизвестная ошибка: " + ex.getMessage(), "Результат",
                                JOptionPane.ERROR_MESSAGE);
                        logwarn.log(Level.FATAL, "Неизвестная ошибка: " + ex.getMessage());
                    }
                });

                form.setSize(975, 360);
                form.setVisible(true);
            }
        });

        //удаление самолёта
        ButtonRemovePlane.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (MaskedTextBoxPosition.getText().isEmpty())
                {
                    return;
                }

                int result = JOptionPane.showConfirmDialog(null,"Удалить объект?","Удаление",
                        JOptionPane.YES_NO_OPTION,JOptionPane.WARNING_MESSAGE);

                if(result == JOptionPane.NO_OPTION)
                {
                    return;
                }

                int pos = Integer.parseInt(MaskedTextBoxPosition.getText());

                try {
                    var deletePlane = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).Delete(pos);

                    if (deletePlane != null) {
                        JOptionPane.showMessageDialog(null, "Объект удалён " + deletePlane);
                        loginfo.log(Level.INFO, "Oбъект удалён: " + deletePlane);
                        UpdateWindow(_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet());
                    } else {
                        JOptionPane.showMessageDialog(null, "Не удалось удалить объект", "Ошибка", JOptionPane.ERROR_MESSAGE);
                        logwarn.log(Level.ERROR, "Не удалось удалить объект " + deletePlane);
                    }
                }
                catch (PlaneNotFoundException ex)
                {
                    JOptionPane.showMessageDialog(null, "Ошибка удаления: " + ex.getMessage(), "Ошибка", JOptionPane.ERROR_MESSAGE);
                    logwarn.log(Level.WARN, "Ошибка удаления: " + ex.getMessage());
                }
                catch (Exception ex)
                {
                    JOptionPane.showMessageDialog(null, "Неизвестная ошибка: " + ex.getMessage(), "Ошибка", JOptionPane.ERROR_MESSAGE);
                    logwarn.log(Level.FATAL, "Ошибка удаления: " + ex.getMessage());
                }
            }
        });

        //показ хранилища
        ButtonShowStorage.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(_mapsCollection == null)
                {
                    return;
                }

                UpdateWindow(_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet());
            }
        });

        //показ карты
        ButtonShowOnMap.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (_mapsCollection == null)
                {
                    return;
                }

                try {
                    UpdateWindow(_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowOnMap());
                } catch (StorageOverflowException ex) {
                    ex.printStackTrace();
                } catch (PlaneNotFoundException ex) {
                    ex.printStackTrace();
                }
            }
        });

        ButtonUp.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(_mapsCollection== null)
                {
                    return;
                }

                PictureBoxPlane.removeAll();

                JLabel imageWithMapAndObject = new JLabel();
                imageWithMapAndObject.setPreferredSize(PictureBoxPlane.getSize());
                imageWithMapAndObject.setMinimumSize(new Dimension(1, 1));
                imageWithMapAndObject.setIcon(new ImageIcon(_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).MoveObject((Direction.Up))));

                PictureBoxPlane.add(imageWithMapAndObject, BorderLayout.CENTER);
                PictureBoxPlane.revalidate();
                PictureBoxPlane.repaint();
            }
        });

        ButtonLeft.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(_mapsCollection == null)
                {
                    return;
                }

                PictureBoxPlane.removeAll();

                JLabel imageWithMapAndObject = new JLabel();
                imageWithMapAndObject.setPreferredSize(PictureBoxPlane.getSize());
                imageWithMapAndObject.setMinimumSize(new Dimension(1, 1));
                imageWithMapAndObject.setIcon(new ImageIcon(_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).MoveObject((Direction.Left))));

                PictureBoxPlane.add(imageWithMapAndObject, BorderLayout.CENTER);
                PictureBoxPlane.revalidate();
                PictureBoxPlane.repaint();
            }
        });

        ButtonDown.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(_mapsCollection == null)
                {
                    return;
                }

                PictureBoxPlane.removeAll();

                JLabel imageWithMapAndObject = new JLabel();
                imageWithMapAndObject.setPreferredSize(PictureBoxPlane.getSize());
                imageWithMapAndObject.setMinimumSize(new Dimension(1, 1));
                imageWithMapAndObject.setIcon(new ImageIcon(_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).MoveObject((Direction.Down))));

                PictureBoxPlane.add(imageWithMapAndObject, BorderLayout.CENTER);
                PictureBoxPlane.revalidate();
                PictureBoxPlane.repaint();
            }
        });

        ButtonRight.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(_mapsCollection == null)
                {
                    return;
                }

                PictureBoxPlane.removeAll();

                JLabel imageWithMapAndObject = new JLabel();
                imageWithMapAndObject.setPreferredSize(PictureBoxPlane.getSize());
                imageWithMapAndObject.setMinimumSize(new Dimension(1, 1));
                imageWithMapAndObject.setIcon(new ImageIcon(_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).MoveObject((Direction.Right))));

                PictureBoxPlane.add(imageWithMapAndObject, BorderLayout.CENTER);
                PictureBoxPlane.revalidate();
                PictureBoxPlane.repaint();
            }
        });

        MaskedTextBoxPosition.addKeyListener(new KeyAdapter() {
            @Override
            public void keyTyped(KeyEvent e) {
                char c = e.getKeyChar();

                if ((c < '0') || (c > '9'))
                {
                    e.consume();
                }
            }
        });

        //сортировка по типу
        ButtonSortByType.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(ListBoxMaps.getSelectedIndex() == -1)
                {
                    return;
                }

                _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).Sort(new PlaneCompareByType());
                UpdateWindow(_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet());
            }
        });

        //сортировка по цвету
        ButtonSortByColor.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(ListBoxMaps.getSelectedIndex() == -1)
                {
                    return;
                }

                _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).Sort(new PlaneCompareByColor());
                UpdateWindow(_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet());
            }
        });

        //обработка нажатия загрузки всех карт
        MenuItemLoadData.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                JFileChooser jfc = new JFileChooser();
                int result = jfc.showOpenDialog(null);

                if (result == JFileChooser.APPROVE_OPTION)
                {
                    try
                    {
                        _mapsCollection.LoadData(jfc.getSelectedFile().getPath());
                        ReloadMaps();
                        JOptionPane.showMessageDialog(null,"Загрузка данных прошла успешно",
                                "Результат", JOptionPane.INFORMATION_MESSAGE);
                        loginfo.log(Level.INFO, "Загрузка карт прошла успешно");
                    }
                    catch(Exception ex)
                    {
                        JOptionPane.showMessageDialog(null, "Ошибка загрузки данных" + ex.getMessage(),
                                "Результат", JOptionPane.ERROR_MESSAGE);
                        logwarn.log(Level.ERROR, "Ошибка загрузки: " + ex.getMessage());
                    }
                }
            }
        });

        //обработка нажатия сохранения всех карт
        MenuItemSaveData.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                JFileChooser jfc = new JFileChooser();
                int result = jfc.showSaveDialog(null);

                if (result == JFileChooser.APPROVE_OPTION)
                {
                    try
                    {
                        _mapsCollection.SaveData(jfc.getSelectedFile().getPath());
                        JOptionPane.showMessageDialog(null,"Сохранение прошло успешно",
                                "Результат", JOptionPane.INFORMATION_MESSAGE);
                        logwarn.log(Level.ERROR,"Сохранение прошло успешно");
                    }
                    catch(Exception ex)
                    {
                        JOptionPane.showMessageDialog(null, "Не сохранилось" + ex.getMessage(),
                                "Результат", JOptionPane.ERROR_MESSAGE);
                        logwarn.log(Level.ERROR,"Ошибка сохранения: " + ex.getMessage());
                    }
                }
            }
        });

        //загрузка одной карты
        MenuItemLoadOneMapData.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                JFileChooser jfc = new JFileChooser();
                int result = jfc.showOpenDialog(null);

                if (result == JFileChooser.APPROVE_OPTION)
                {
                    try {
                        if (_mapsCollection.LoadOneData(jfc.getSelectedFile().getPath())) {
                            ReloadMaps();
                            JOptionPane.showMessageDialog(null, "Загрузка данных прошла успешно",
                                    "Результат", JOptionPane.INFORMATION_MESSAGE);
                        } else {
                            JOptionPane.showMessageDialog(null, "Ошибка загрузки данных",
                                    "Результат", JOptionPane.ERROR_MESSAGE);
                        }
                    }
                    catch(Exception ex)
                    {
                        throw new RuntimeException(ex);
                    }
                }
            }
        });

        //сохранение одной карты
        MenuItemSaveOneMapData.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                JFileChooser jfc = new JFileChooser();
                int result = jfc.showSaveDialog(null);

                if (result == JFileChooser.APPROVE_OPTION)
                {
                    try {
                        _mapsCollection.SaveOneData(jfc.getSelectedFile().getPath(), ListBoxMaps.getSelectedValue().toString());
                        JOptionPane.showMessageDialog(null, "Сохранение прошло успешно",
                                "Результат", JOptionPane.INFORMATION_MESSAGE);
                    }
                    catch(Exception ex)
                    {
                        JOptionPane.showMessageDialog(null, "Не сохранилось " + ex.getMessage(),
                                "Результат", JOptionPane.ERROR_MESSAGE);
                        throw new RuntimeException(ex);
                    }
                }
            }
        });
    }

    private void createUIComponents()
    {
        DefaultListModel<String> defListMod = new DefaultListModel<String>();
        ListBoxMaps = new JList(defListMod);
    }
}
