import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.util.Random;

public class FormPlane extends JDialog
{
    public JPanel MainPanel;
    private JButton ButtonCreate;
    private JButton ButtonLeft;
    private JButton ButtonDown;
    private JButton ButtonRight;
    private JButton ButtonUp;
    private JToolBar StatusStrip;
    private JPanel PictureBoxPlane;
    private JButton ButtonCreateModif;
    private JButton ButtonSelectPlane;
    private JLabel LabelSpeed = new JLabel();
    private JLabel LabelWeight = new JLabel();
    private JLabel LabelColor = new JLabel();

    protected DrawingPlane _plane;
    protected DrawingPlane _selectedPlane;
    private int PictureBoxPlaneWidth;
    private int PictureBoxPlaneHeight;

    public DrawingPlane GetSelectedShip()
    {
        return _selectedPlane;
    }

    //метод отрисовки
    public void Draw(DrawingPlane _plane) {
        PictureBoxPlane.removeAll();
        BufferedImage bmp = new BufferedImage(PictureBoxPlane.getWidth(), PictureBoxPlane.getHeight(), BufferedImage.TYPE_INT_RGB);
        Graphics gr = bmp.getGraphics();

        gr.setColor(new Color(238, 238, 238));
        gr.fillRect(0, 0, PictureBoxPlane.getWidth(), PictureBoxPlane.getHeight());

        if (_plane.GetPlane() != null) {
            _plane.DrawTransport(gr);
            JLabel imageOfLogo = new JLabel();
            imageOfLogo.setPreferredSize(PictureBoxPlane.getSize());
            imageOfLogo.setMinimumSize(new Dimension(1, 1));
            imageOfLogo.setIcon(new ImageIcon(bmp));
            PictureBoxPlane.add(imageOfLogo, BorderLayout.CENTER);
        }

        validate();
    }

    //создание всплывающего окна (первый конструктор)
    public FormPlane()
    {
        super(new Frame("Аэробус"));
        CreateWindow();
        setModal(true);
        getContentPane().add(MainPanel);
    }

    //второй конструктор
    protected FormPlane(DrawningObjectPlane plane)
    {
        super(new Frame("Аэробус"));
        CreateWindow();
        setModal(true);
        _plane = plane.GetDrawingObjectPlane();
        SetData();
    }

    public DrawingPlane GetSelectedPlane()
    {
        return _selectedPlane;
    }

    public void SetData()
    {
        Random rnd = new Random();
        _plane.SetPosition(rnd.nextInt(10, 100), rnd.nextInt(10, 100),
                PictureBoxPlane.getWidth(), PictureBoxPlane.getHeight());

        LabelSpeed.setText("Скорость: " + _plane.GetPlane().GetSpeed() + " ");
        LabelWeight.setText("Вес: " + _plane.GetPlane().GetWeight() + " ");
        LabelColor.setText("Цвет: r = " + _plane.GetPlane().GetColor().getRed() + " g = " + _plane.GetPlane().GetColor().getGreen() +
                " b = " + _plane.GetPlane().GetColor().getBlue());
        PictureBoxPlane.add(_plane, BorderLayout.CENTER);
    }

    public void CreateWindow()
    {
        //создание строки отображения скорости, веса и цвета объекта
        Box LableBox = Box.createHorizontalBox();
        LableBox.setMinimumSize(new Dimension(1, 20));
        LableBox.add(LabelSpeed);
        LableBox.add(LabelWeight);
        LableBox.add(LabelColor);
        StatusStrip.add(LableBox);

        PictureBoxPlane.setBackground(Color.WHITE);
        setPreferredSize(new Dimension(1000, 700));
        getContentPane().add(MainPanel);

        try
        {
            Image img = ImageIO.read(getClass().getResource("resourses/Up.png"));
            ButtonUp.setIcon(new ImageIcon(img));
            img = ImageIO.read(getClass().getResource("resourses/Left.png"));
            ButtonLeft.setIcon(new ImageIcon(img));
            img = ImageIO.read(getClass().getResource("resourses/Down.png"));
            ButtonDown.setIcon(new ImageIcon(img));
            img = ImageIO.read(getClass().getResource("resourses/Right.png"));
            ButtonRight.setIcon(new ImageIcon(img));
        }
        catch (Exception ex)
        {
            System.out.println(ex.getMessage());
        }

        _selectedPlane = _plane;

        ButtonCreate.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                try
                {
                    PictureBoxPlane.remove(_plane);
                }
                catch (Exception c){ }

                Random rnd = new Random();
                Color colorSimple = JColorChooser.showDialog(null, "Выберите цвет", null);

                _plane = new DrawingPlane(rnd.nextInt(100, 300), rnd.nextInt(1000, 2000),
                        colorSimple, rnd.nextInt(3));

                SetData();

                Draw(_plane);
            }
        });

        ButtonCreateModif.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                try
                {
                    PictureBoxPlane.remove(_plane);
                }
                catch (Exception c){ }

                Random rnd = new Random();
                Color colorSimple = JColorChooser.showDialog(null, "Выберите цвет", null);
                Color colorModif = JColorChooser.showDialog(null, "Выберите цвет", null);

                _plane = new DrawingAirbus(rnd.nextInt(100, 300), rnd.nextInt(1000, 2000),
                        colorSimple, rnd.nextInt(3), colorModif, rnd.nextBoolean(), rnd.nextBoolean());

                SetData();

                Draw(_plane);
            }
        });

        ButtonSelectPlane.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                _selectedPlane = _plane;
                dispose();
            }
        });

        PictureBoxPlane.addComponentListener(new ComponentAdapter() {
            @Override
            public void componentResized(ComponentEvent e) {
                super.componentResized(e);

                if(_plane == null)
                {
                    return;
                }

                _plane.ChangeBorders(PictureBoxPlane.getWidth(), PictureBoxPlane.getHeight());

                try
                {
                    PictureBoxPlane.remove(_plane);
                }
                catch (Exception c){ }

                PictureBoxPlane.add(_plane, BorderLayout.CENTER);
            }
        });

        ButtonUp.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                _plane.MoveTransport(Direction.Up);
                PictureBoxPlane.revalidate();
                Draw(_plane);
            }
        });

        ButtonLeft.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                _plane.MoveTransport(Direction.Left);
                PictureBoxPlane.revalidate();
                Draw(_plane);
            }
        });

        ButtonDown.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                _plane.MoveTransport(Direction.Down);
                PictureBoxPlane.revalidate();
                Draw(_plane);
            }
        });

        ButtonRight.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                _plane.MoveTransport(Direction.Right);
                PictureBoxPlane.revalidate();
                Draw(_plane);
            }
        });

        //кнопка подтверждение выбора
        ButtonSelectPlane.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                _selectedPlane = _plane;
                dispose();
            }
        });
    }
}
