import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.SortedSet;

public class MapWithSetPlanesGeneric <T extends IDrawningObject, U extends AbstractMap>
{
    //ширина окна отрисовки
    private final int _pictureWidth;

    //высота окна отрисовки
    private final int _pictureHeight;

    //размер занимаемого объектом места (ширина)
    private final int _placeSizeWidth = 210;

    //размер занимаемого объектом места (высота)
    private final int _placeSizeHeight = 90;

    //набор объектов
    private SetPlanesGeneric<T> _setPlanes;

    //набор удалённых объектов (двусвязный список)
    public final LinkedList<T> _deletePlane;

    //карта
    private U _map;

    //конструктор
    public MapWithSetPlanesGeneric(int picWidth, int picHeight, U map)
    {
        int width = picWidth / _placeSizeWidth;
        int height = picHeight / _placeSizeHeight;
        _setPlanes = new SetPlanesGeneric<T>(10);
        _deletePlane = new LinkedList<>();
        _pictureWidth = picWidth;
        _pictureHeight = picHeight;
        _map = map;
    }

    public int Add(T plane) throws StorageOverflowException
    {
        return _setPlanes.Insert(plane);
    }

    public T Delete(int position) throws PlaneNotFoundException {
        T temp = _setPlanes.Remove(position);
        _deletePlane.add(temp);

        return temp;
    }

    //вывод всего набора объектов
    public BufferedImage ShowSet()
    {
        BufferedImage bmp = new BufferedImage(_pictureWidth, _pictureHeight, BufferedImage.TYPE_INT_RGB);
        Graphics gr = bmp.getGraphics();
        DrawBackground(gr);
        DrawPlanes(gr);

        return bmp;
    }

    //просмотр объекта на карте
    public BufferedImage ShowOnMap() throws StorageOverflowException, PlaneNotFoundException {
        Shaking();

        for(int i = 0; i < _setPlanes.Count(); i++)
        {
            var plane = _setPlanes.Get(i);

            if(plane != null)
            {
                return _map.CreateMap(_pictureWidth, _pictureHeight, plane);
            }
        }

        return new BufferedImage(_pictureWidth, _pictureHeight, BufferedImage.TYPE_INT_RGB);
    }

    //пеермещение объекта по карте
    public BufferedImage MoveObject(Direction direction)
    {
        if(_map != null)
        {
            return _map.MoveObject(direction);
        }

        return new BufferedImage(_pictureWidth, _pictureHeight, BufferedImage.TYPE_INT_RGB);
    }

    //получение данных в виде строки
    public String GetData(char separatorType, char separatorData)
    {
        String data = String.format("%s%c", _map.getClass().getName(), separatorType);

        for (var plane : _setPlanes)
        {
            data += String.format("%s%c", plane.GetInfo(), separatorData);
        }

        return data;
    }

    //Загрузка списка из массива строк
    public void LoadData(String[] records) throws StorageOverflowException {
        for (var rec : records)
        {
            _setPlanes.Insert((T)DrawningObjectPlane.Create(rec));
        }
    }

    //сортировка
    public void Sort(Comparator<T> comparer)
    {
        _setPlanes.SortSet(comparer);
    }

    //"взламываем" набор, чтобы все элементы оказались в начале
    private void Shaking() throws StorageOverflowException, PlaneNotFoundException {
        int j = _setPlanes.Count() - 1;

        for (int i = 0; i < _setPlanes.Count(); i++)
        {
            if (_setPlanes.Get(i) == null)
            {
                for (; j > i; j--)
                {
                    var plane = _setPlanes.Get(j);

                    if (plane != null)
                    {
                        _setPlanes.Insert(plane, i);
                        _setPlanes.Remove(j);
                        break;
                    }
                }

                if (j <= i)
                {
                    return;
                }
            }
        }
    }

    //метод отрисовки фона
    public void DrawBackground(Graphics g)
    {
        Graphics2D g2d = (Graphics2D) g;

        //заливаем область в цвет бетона
        g2d.setPaint(Color.LIGHT_GRAY);
        g2d.fillRect(0, 0, _pictureWidth, _pictureHeight);

        g2d.setStroke(new BasicStroke(3));
        g2d.setColor(Color.BLACK);

        for(int i = 0; i < _pictureWidth / _placeSizeWidth - 1; i++)
        {
            //линия разметки места
            for(int j = 2; j < _pictureHeight / _placeSizeHeight + 1; ++j)
            {
                g2d.drawLine(i * _placeSizeWidth + 5, j * _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth / 2 + 5, j * _placeSizeHeight);
            }

            g2d.drawLine(i * _placeSizeWidth + 5, _placeSizeHeight * 2, i * _placeSizeWidth + 5, (_pictureHeight / _placeSizeHeight) * _placeSizeHeight);
        }

        //отрисовка разметки взлётной полосы
        g2d.setColor(Color.DARK_GRAY);
        g2d.fillRect(_placeSizeWidth * 2 + 35, 0, 175, _pictureHeight);

        g2d.setStroke(new BasicStroke(5));
        g2d.setColor(Color.WHITE);
        g2d.drawLine(_placeSizeWidth * 2 + 210, 0, _placeSizeWidth * 2 + 210, _pictureHeight);
        g2d.drawLine(_placeSizeWidth * 2 + 35, 0, _placeSizeWidth * 2 + 35, _pictureHeight);
        g2d.drawLine(_placeSizeWidth * 2 + 215, 0, _placeSizeWidth * 2 + 215, _pictureHeight);
        g2d.drawLine(_placeSizeWidth * 2 + 30, 0, _placeSizeWidth * 2 + 30, _pictureHeight);

        g2d.setStroke(new BasicStroke(5));
        g2d.setColor(Color.WHITE);
        for (int i = 0; i < _pictureHeight / _placeSizeHeight; ++i)
        {
            g2d.drawLine(_placeSizeWidth * 2 + 125, 20 + i * _placeSizeHeight, _placeSizeWidth * 2 + 125, (i + 1) * _placeSizeHeight - 20);
        }

        g2d.setColor(new Color(0xFF, 0x45, 0x00));
        for(int i = 0; i < _pictureHeight / 20; i++)
        {
            g2d.drawLine(_placeSizeWidth * 2 + 15, 20 + i * _placeSizeHeight / 2, _placeSizeWidth * 2 + 15, (i + 1) * _placeSizeHeight / 2 - 20);
            g2d.drawLine(_placeSizeWidth * 3 + 20, 20 + i * _placeSizeHeight / 2, _placeSizeWidth * 3 + 20, (i + 1) * _placeSizeHeight / 2 - 20);
        }

        //отрисовка сочков
        for(int i = 1; i < 6; i++)
        {
            g2d.setColor(new Color(0xFF, 0xB6, 0xC1));
            int[] xPoint = {(i * 70 - 10) + 45, i * 70 - 10, i * 70 - 10};
            int[] yPoint = {30, 50, 10};
            g2d.fillPolygon(xPoint, yPoint, xPoint.length);

            g2d.setStroke(new BasicStroke(3));
            g2d.setColor(Color.BLACK);
            g2d.drawLine(i * 70 - 10, 10, i * 70 - 10, 80);
            g2d.drawLine(i * 70 - 10, 10, (i * 70 - 10) + 45, 30);
            g2d.drawLine(i * 70 - 10, 50, (i * 70 - 10) + 45, 30);
        }
    }

    //метод прорисовки объеков
    public void DrawPlanes(Graphics g)
    {
        int position = 0;
        int currentWidth = 1;
        int currentHeight = 6;

        for (int i = 0; i < _setPlanes.Count(); i++)
        {
            if(_setPlanes.Get(i) != null)
            {
                _setPlanes.Get(i).SetObject(currentWidth * _placeSizeWidth + 20, currentHeight * _placeSizeHeight + 20, _pictureWidth, _pictureHeight);
                _setPlanes.Get(i).DrawningObject(g);
            }

            if(position % 2 == 0)
            {
                position++;
                currentWidth--;
            }
            else
            {
                position = 0;
                currentWidth = 1;
                currentHeight--;
            }
        }
    }

    //очистка и ключа, и значения по ключу
    public void Clear()
    {
        _setPlanes = null;
    }

    public T GetPlaneInList(int index)
    {
        return _setPlanes.Get(index);
    }

    public T GetPlaneInDelete()
    {
        if(_deletePlane.isEmpty())
        {
            return null;
        }

        return _deletePlane.pop();
    }
}
